<?php

namespace Drupal\reaction\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\reaction\Controller\ReactionController;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Path\PathValidatorInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Plugin implementation of the 'reaction_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "reaction_formatter",
 *   label = @Translation("Reaction field formatter"),
 *   field_types = {
 *     "reaction"
 *   }
 * )
 */
class ReactionFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * @var AccountInterface $account
   */
  protected $account;

  /**
   * The path validator service.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected $pathValidator;
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id, $plugin_definition, $configuration['field_definition'], $configuration['settings'], $configuration['label'], $configuration['view_mode'], $configuration['third_party_settings'], $container->get('path.validator'), $container->get('current_user'), $container->get('database')
    );
  }

  /**
   * Constructs a new LinkFormatter.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\Core\Path\PathValidatorInterface $path_validator
   *   The path validator service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, PathValidatorInterface $path_validator, AccountInterface $account) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->pathValidator = $path_validator;
    $this->account = $account;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Implement default settings.
      'reaction_style' => 'google',
      'reaction_position' => 'horizontal',
      'reaction_css_class_name' => 'reaction',
      'reaction_layout_style' => 'simple_layout',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */

    public function settingsForm(array $form, FormStateInterface $form_state) {
      global $base_url;
      $elements = parent::settingsForm($form, $form_state);
      $path = drupal_get_path('module', 'reaction');
      if ($handle = opendir($path . '/vendor/krknth/reaction/')) {
        while (false !== ($entry = readdir($handle))) {
          if ($entry != "." && $entry != ".." && $entry != "." && $entry != ".git" && $entry != "Readme.txt" && $entry != "composer.json") {
            $reaction_style_option[$entry] = ucfirst($entry);
          }
        }
        closedir($handle);
      }
      $imageOptions = array(
        "simple_layout" => '<img src = ' . $base_url . '/' . $path . '/styles/icons/simple_layout.png>',
        "fancy_layout" => '<img src = ' . $base_url . '/' . $path . '/styles/icons/fancy_layout.png>',
      );
      $elements['reaction_style'] = [
        '#type' => 'select',
        '#title' => t('Select the type of style'),
        '#options' => $reaction_style_option,
        '#default_value' => $this->getSetting('reaction_style'),
      ];
      $elements['reaction_position'] = array(
        '#type' => 'select',
        '#title' => t('Select the position'),
        '#options' => array('vertical' => 'Vertical', 'horizontal' => 'Horizontal'),
        '#default_value' => $this->getSetting('reaction_position'),
      );
      $elements['reaction_css_class_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Css Class name'),
        '#default_value' => $this->getSetting('reaction_css_class_name'),
      );
      $elements['reaction_layout_style'] = array(
        '#type' => 'radios',
        '#options' => $imageOptions,
        '#title' => t('Choose the Layout'),
        '#default_value' => $this->getSetting('reaction_layout_style'),
      );
      return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();

    $settings = $this->getSettings();
    if (!empty($settings['reaction_style'])) {
      $summary[] = t('Reaction is styled to @style', array('@style' => $settings['reaction_style']));
    }
    else {
      $summary[] = t('Reaction is not styled');
    }
    if (!empty($settings['reaction_position'])) {
      $summary[] = t('Reaction is position at @position', array('@position' => $settings['reaction_position']));
    }
    else {
      $summary[] = t('Reaction is not positioned');
    }
    if (!empty($settings['reaction_css_class_name'])) {
      $summary[] = t('Reaction\'s CSS class name is @class_name', array('@class_name' => $settings['reaction_css_class_name']));
    }
    else {
      $summary[] = t('No css class name');
    }
    if (!empty($settings['reaction_layout_style'])) {
      if($settings['reaction_layout_style'] == 'simple_layout') {
        $layout = 'Simple Layout';
      }
      elseif($settings['reaction_layout_style'] == 'fancy_layout')
      {
        $layout = 'Fancy Layout';
      }
      $summary[] = t('Reaction\'s layout is @layout', array('@layout' => $layout));
    }
    else {
      $summary[] = t('No layout is selected');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    global $base_url;
    $elements = [];
    if ($this->account && $this->account->hasPermission('display reaction formatter')) {
      $module_path = drupal_get_path('module', 'reaction');
      $reacted = FALSE;
      $user_reacted_emoji_class = '';
      $style_setting = $this->getSetting('reaction_style');
      $layout_style = $this->getSetting('reaction_layout_style');
      $entity = $items->getEntity();
      $entity_type = $entity->getEntityTypeId();
      $bundle = $entity->bundle();
      $entity_id = $entity->id();
      $reaction_emoji_display = $this->getFieldSetting('reaction_style');
      $ip = \Drupal::request()->getClientIp();
      $user_id = \Drupal::currentUser()->id();
      $result = ReactionController::Checking_reacted($entity_id, $user_id, $ip);
      if (count($result) >= 1) {
        $reacted = TRUE;
        $user_reacted_emoji_class = 'user_reacted_emoji';
      }
      //To get the total count which we are using in calculate percentage
      $total_count = ReactionController::get_emoji_total_count($reaction_emoji_display, $entity_id);
      foreach ($reaction_emoji_display as $emoji) {
        $count = ReactionController::_get_count_reaction($entity_id, $emoji);
        $url_data = [
          'entity_type' => $entity_type,
          'bundle' => $bundle,
          'entity_id' => $entity_id,
          'expression' => $emoji,
          'ip' => $ip,
          'reacted_count' => $count,
          'reaction_emoji_display' => $reaction_emoji_display,
          'style_setting' => $style_setting,
          'layout_style' => $layout_style
        ];
        $json_data = ReactionController::base64url_encode(json_encode($url_data));
        $url = Url::fromRoute(
          'ajax_link.response', ['url_encode_data' => $json_data]
        );
        $count = ReactionController::_get_count_reaction($entity_id, $emoji);
        $percentage_calculate = 100 / $total_count;

        if (!is_numeric($emoji)) {
          $image[] = array(
            'image_path' => $base_url . '/' . $module_path . '/vendor/krknth/reaction/' . $style_setting . '/' . $emoji . '.png',
            'emoji_name' => $emoji,
            'image_url' => $url,
            'user_reacted_emoji' => $user_reacted_emoji_class,
            'percentage' => is_nan(round($count * $percentage_calculate)) ? 0 : round($count * $percentage_calculate),
            'reacted_count' => $count,
          );
        }
      }

      $elements[] = array(
        '#theme' => 'reaction_formatter',
        '#style' => $style_setting,
        '#image' => $image,
        '#layout_style' => $layout_style,
        '#reacted' => $reacted,
      );
      \Drupal::service('page_cache_kill_switch')->trigger();
      //To disable the cache for authenticate user
      $elements['#cache']['max-age'] = 0;
    }
    $elements['#attached']['library'][] = 'core/drupal.ajax';
    $elements['#attached']['library'][] = 'reaction/reaction';
    return $elements;
  }
}
